#define BOOST_TEST_MODULE "testcase for Source class and Source1, Source2 class"
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <string>
#include <boost/shared_ptr.hpp>
#include <harkio2/harkconfig/Source.hpp>
using namespace harkio2;
using namespace harkio2::harkconfig;
using namespace boost;
using namespace std;

typedef boost::shared_ptr<Source> sp;
typedef boost::shared_ptr<Source1> sp1;
typedef boost::shared_ptr<Source2> sp2;

BOOST_AUTO_TEST_CASE(versioning_test)
{
    sp s0(new Source1());
    BOOST_CHECK_EQUAL(s0->GetVersion(), 1);

    sp s1(new Source2());
    BOOST_CHECK_EQUAL(s1->GetVersion(), 2);
}

BOOST_AUTO_TEST_CASE(getter_setter_test)
{
    sp s0(new Source1());

    BOOST_CHECK_EQUAL(s0->GetVersion(), 1);

    sp1 s1 = dynamic_pointer_cast<Source1>(s0);

    BOOST_CHECK_EQUAL(s1->GetVersion(), 1);

    Vector3<float> v(10, 20, 30);
    s1->SetPos(v);
    s1->SetFile("hoge.wav");
    s1->SetCutStart(123);
    s1->SetCutEnd(67);

    BOOST_CHECK(s1->GetPos() == Vector3<float>(10, 20, 30));
    BOOST_CHECK_EQUAL(s1->GetFile(), "hoge.wav");
    BOOST_CHECK_EQUAL(s1->GetCutStart(), 123);
    BOOST_CHECK_EQUAL(s1->GetCutEnd(), 67);
}

BOOST_AUTO_TEST_CASE(getter_setter_test2)
{
    sp s0(new Source2());

    BOOST_CHECK_EQUAL(s0->GetVersion(), 2);

    sp2 s2 = dynamic_pointer_cast<Source2>(s0);

    BOOST_CHECK_EQUAL(s2->GetVersion(), 2);

    Position<float> p = MakeDirectionFromVector(Vector3<float>(1, 1, 0));
    vector<File1> files(1);
    vector<unsigned int> indices(1);
    indices.back() = 10;
    File1& f = files.back();
    f.SetName("foo.bar");
    f.SetMicIndices(indices);

    s2->SetPosition(p);
    s2->SetFiles(files);
    s2->SetCutStart(2);
    s2->SetCutStart(3);
    s2->SetComment("comment string!\nline 2");

    BOOST_CHECK(s2->GetPosition().GetUnitVector() == Normalize(Vector3<float>(1, 1, 0)));
    BOOST_CHECK_EQUAL(s2->GetFiles().size(), 1);
    BOOST_CHECK_EQUAL(s2->GetFiles().back().GetName(), "foo.bar");
    BOOST_CHECK_EQUAL(s2->GetFiles().back().GetMicIndices().size(), 1);
    BOOST_CHECK_EQUAL(s2->GetFiles().back().GetMicIndices().back(), 10);
    BOOST_CHECK_EQUAL(s2->GetComment(), "comment string!\nline 2");
}

struct NotInitializedCheck
{
    NotInitializedCheck(const std::string& msg)
        : msg(msg)
    {}

    bool operator ()(const harkio2::NotInitialized& e)
    {
        return msg == e.what();
    }

private:
    std::string msg;
};

BOOST_AUTO_TEST_CASE(exception_test)
{
    sp1 s1(new Source1());

    // Checks whether GetXXX() throws a NotInitialized exception.
    // Now GetXXX must throw it.
    BOOST_CHECK_THROW(s1->GetPos(), NotInitialized);
    BOOST_CHECK_THROW(s1->GetFile(), NotInitialized);
    BOOST_CHECK_THROW(s1->GetCutStart(), NotInitialized);
    BOOST_CHECK_THROW(s1->GetCutEnd(), NotInitialized);

    s1->SetPos(Vector3<float>(1, 2, 3));

    // Now that we set a value to s1->pos, GetPos must NOT throw any exception.
    BOOST_CHECK_NO_THROW(s1->GetPos());
    BOOST_CHECK_THROW(s1->GetFile(), NotInitialized);
    BOOST_CHECK_THROW(s1->GetCutStart(), NotInitialized);
    BOOST_CHECK_THROW(s1->GetCutEnd(), NotInitialized);

    s1->SetFile("hogehoge");
    s1->SetCutStart(123);
    s1->SetCutEnd(67);

    BOOST_CHECK_NO_THROW(s1->GetPos());
    BOOST_CHECK_NO_THROW(s1->GetFile());
    BOOST_CHECK_NO_THROW(s1->GetCutStart());
    BOOST_CHECK_NO_THROW(s1->GetCutEnd());

    sp2 s2(new Source2());

    // Checks whether GetXXX() throws a NotInitialized exception.
    // Now GetXXX must throw it.
    BOOST_CHECK_THROW(s2->GetPosition(), NotInitialized);
    BOOST_CHECK_THROW(s2->GetFiles(), NotInitialized);
    BOOST_CHECK_THROW(s2->GetCutStart(), NotInitialized);
    BOOST_CHECK_THROW(s2->GetCutEnd(), NotInitialized);
    BOOST_CHECK_THROW(s2->GetComment(), NotInitialized);

    s2->SetPosition(MakeDirectionFromPolar<float>(3.14 / 2, 3.14 / 2));

    BOOST_CHECK_NO_THROW(s2->GetPosition());
    BOOST_CHECK_THROW(s2->GetFiles(), NotInitialized);
    BOOST_CHECK_THROW(s2->GetCutStart(), NotInitialized);
    BOOST_CHECK_THROW(s2->GetCutEnd(), NotInitialized);
    BOOST_CHECK_THROW(s2->GetComment(), NotInitialized);

    s2->SetFiles(vector<File1>());
    s2->SetCutStart(123);
    s2->SetCutEnd(67);

    BOOST_CHECK_NO_THROW(s2->GetPosition());
    BOOST_CHECK_NO_THROW(s2->GetFiles());
    BOOST_CHECK_NO_THROW(s2->GetCutStart());
    BOOST_CHECK_NO_THROW(s2->GetCutEnd());
    BOOST_CHECK_THROW(s2->GetComment(), NotInitialized);

    s2->SetComment("comment");

    BOOST_CHECK_NO_THROW(s2->GetComment());
}
